from mpu6050 import mpu6050
import time
import numpy as np
from scipy import signal

import matplotlib.pyplot as plt

from subprocess import call
import paho.mqtt.client as paho

#-----Funcion tras publicar en mqtt-----#
def on_publish(client,userdata,result):
    #print (result)
    pass

#-----Filtros-----#
def median_filter(data, f_size):
    lgth, num_signal=data.shape
    f_data=np.zeros([lgth, num_signal])
    for i in range(num_signal):
        f_data[:,i]=signal.medfilt(data[:,i], f_size)
    return f_data

def freq_filter(data, f_size, cutoff):
	lgth, num_signal=data.shape
	f_data=np.zeros([lgth, num_signal])
	lpf=signal.firwin(f_size, cutoff, window='hamming')
	for i in range(num_signal):
		f_data[:,i]=signal.convolve(data[:,i], lpf, mode='same')
	return f_data
#-----Otros-----#
def string_format(data):
    strdata = ""
    for i in range(len(data)):
        strdata = strdata + str(data[i][0]) + " " + str(data[i][1]) + " " + str(data[i][2]) + "\n"
    return strdata

def plot_lines(data, fs, title):
	num_rows, num_cols=data.shape
	if num_cols!=3:
		raise ValueError('Not 3D data')
	fig, ax=plt.subplots()
	labels=['x','y','z']
	color_map=['r', 'g', 'b']
	index=np.arange(num_rows)/fs
	for i in range(num_cols):
		plt.plot(index, data[:,i], color_map[i], label=labels[i])
	#plt.set_xlim([0,num_rows/fs])
	#plt.set_xlabel('Time [sec]')
	#plt.set_title('Time domain: '+title)
	plt.legend()
	plt.savefig(title)
	plt.clf()

#---- Main -----#
broker = "192.168.0.21"
port = 1883

client1 = paho.Client("control1")
client1.on_publish = on_publish
client1.connect(broker,port)

sensor = mpu6050(0x68)
print (sensor.read_accel_range())
print (sensor.read_gyro_range())

dataset = []
while True:
    time.sleep(0.001)
    ad = sensor.get_accel_data() #datos del acelerometro
    gd = sensor.get_accel_data() #datos del giroscopio
    #ax, ay, az = (ad['x'], ad['y'], ad['z'])
    dataset.append((ad['x'], ad['y'], ad['z']))
    if(len(dataset) >= 1000):
        dataset = np.array(dataset)
        median_data=median_filter(dataset, 35) #El segundo dato es el tamano de la ventana, por defecto es 155 pero tengo que ajustarlo a uno que sea el indicado
        lpf_data=freq_filter(median_data, 35, 10.0/512)
        #print(lpf_data.shape)
        ret = client1.publish("Accelerometer", string_format(lpf_data))
        plot_lines(dataset, 512, "Datos sin filtrar")
        plot_lines(median_data, 512, "Datos filtrados median")
        plot_lines(lpf_data, 512, "Datos filtrados all filt")
        dataset = []
        exit(0)
