from mpu6050 import mpu6050
import time
import numpy as np
from scipy import signal

from subprocess import call
import paho.mqtt.client as paho
import os

import matplotlib.pyplot as plt
import re
from datetime import datetime

#-----Funcion tras publicar en mqtt-----#
def on_publish(client,userdata,result):
    #print (result)
    pass

#-----Agregation-----# Source: https://www.ncbi.nlm.nih.gov/pmc/articles/PMC5051724/
def MAD(data): #Mean Amplitude Deviation
    n = float(len(data))
    r = 0.0
    mad = 0.0
    de = 0.0
    for i in data:
        r = r + np.linalg.norm(i)
    r = r / n
    for i in data:
        aux = np.linalg.norm(i) - r
        mad = mad + abs(aux)
        de = de + aux ** 2
    return (mad / n, de / n)

def ENMO(vector):
    return (np.linalg.norm(vector)-9.80665)


def testing_fast(dataset, f1, f2, f3, times):
    sensor = mpu6050(0x68)
    while(times):
        time.sleep(0.01)
        ad = sensor.get_accel_data() #datos del acelerometro
        enmo = ENMO(np.array([ad['x'], ad['y'], ad['z']]))
        f2.write("%f %f %f\n" % (ad['x'], ad['y'], ad['z']))
        f3.write("%f \n" % (enmo))
        dataset.append((ad['x'], ad['y'], ad['z']))
        if(len(dataset) >= 500):
            dataset = np.array(dataset)
            mad, de = MAD(dataset)
            send = str(mad) + " " + str(de) + "\n"
            f1.write(send)
            times = times - 1
            dataset = []
    f1.close()
    f2.close()
    f3.close()


if __name__ == "__main__":
    try: #Setting configuration
        f = open("config",'r')
        for l in f:
            broker, port, test, nodeID = l.strip().split()
            port = int(port)
            test = int(test)
            nodeID = int(nodeID)
            break
        f.close()
    except IOError: #Default configuration
        broker = "192.168.0.21"
        port = 1883
        test = 0
        nodeID = 1

    client1 = paho.Client("control1")
    client1.on_publish = on_publish
    client1.connect(broker,port)

    #----------------- Testing architecture -----------------#
    if(test):
        sizearq = []
        sizenoarq = []
        sizeanotherarq = []

        times = 1
        dataset = []
        arq = open("arq1.txt",'w')
        noarq = open("noarq1.txt",'w')
        anotherarq = open("anotherarq1.txt",'w')
        testing_fast(dataset, arq, noarq, anotherarq, times)
        sizearq.append(os.stat("arq1.txt").st_size)
        sizenoarq.append(os.stat("noarq1.txt").st_size)
        sizeanotherarq.append(os.stat("anotherarq1.txt").st_size)

        times = 10
        dataset = []
        arq = open("arq10.txt",'w')
        noarq = open("noarq10.txt",'w')
        anotherarq = open("anotherarq10.txt",'w')
        testing_fast(dataset, arq, noarq, anotherarq, times)
        sizearq.append(os.stat("arq10.txt").st_size)
        sizenoarq.append(os.stat("noarq10.txt").st_size)
        sizeanotherarq.append(os.stat("anotherarq10.txt").st_size)

        times = 20
        dataset = []
        arq = open("arq20.txt",'w')
        noarq = open("noarq20.txt",'w')
        anotherarq = open("anotherarq20.txt",'w')
        testing_fast(dataset, arq, noarq, anotherarq, times)
        sizearq.append(os.stat("arq20.txt").st_size)
        sizenoarq.append(os.stat("noarq20.txt").st_size)
        sizeanotherarq.append(os.stat("anotherarq20.txt").st_size)

        times = 30
        dataset = []
        arq = open("arq30.txt",'w')
        noarq = open("noarq30.txt",'w')
        anotherarq = open("anotherarq30.txt",'w')
        testing_fast(dataset, arq, noarq, anotherarq, times)
        sizearq.append(os.stat("arq30.txt").st_size)
        sizenoarq.append(os.stat("noarq30.txt").st_size)
        sizeanotherarq.append(os.stat("anotherarq30.txt").st_size)

        times = 40
        dataset = []
        arq = open("arq40.txt",'w')
        noarq = open("noarq40.txt",'w')
        anotherarq = open("anotherarq40.txt",'w')
        testing_fast(dataset, arq, noarq, anotherarq, times)
        sizearq.append(os.stat("arq40.txt").st_size)
        sizenoarq.append(os.stat("noarq40.txt").st_size)
        sizeanotherarq.append(os.stat("anotherarq40.txt").st_size)

        times = 50
        dataset = []
        arq = open("arq50.txt",'w')
        noarq = open("noarq50.txt",'w')
        anotherarq = open("anotherarq50.txt",'w')
        testing_fast(dataset, arq, noarq, anotherarq, times)
        sizearq.append(os.stat("arq50.txt").st_size)
        sizenoarq.append(os.stat("noarq50.txt").st_size)
        sizeanotherarq.append(os.stat("anotherarq50.txt").st_size)

        x = [1, 10, 20, 30, 40, 50]
        plt.plot(x, sizearq, 'b-', label='Arquitecture')
        plt.plot(x, sizenoarq, 'r-', label='No Arquitecture')
        plt.plot(x, sizeanotherarq, 'g-', label='Another Arquitecture')
        plt.title('File Size v/s Iteration')
        plt.xlabel('Iterations')
        plt.ylabel('Size [b]')
        plt.savefig('Size.png')
        plt.clf()

        exit(0)

    #----------------- End testing architecture -----------------#
    sensor = mpu6050(0x68)
    #print (sensor.read_accel_range())
    #print (sensor.read_gyro_range())

    dataset = []

    while True:
        time.sleep(0.01)
        ad = sensor.get_accel_data() #datos del acelerometro
        #gd = sensor.get_accel_data() #datos del giroscopio
        dataset.append((ad['x'], ad['y'], ad['z']))
        if(len(dataset) >= 500):
            dataset = np.array(dataset)
            mad, de = MAD(dataset)
            send = str(mad) + " " + str(de) + " " + str(nodeID)
            ret = client1.publish("Accelerometer", send)
            dataset = []
