import time
import numpy as np
import cv2
import datetime
from math import atan2, cos, sin, sqrt, pi
import paho.mqtt.client as paho


def getOrientation(pts): #Obtiene orientación y realiza PCA
    sz = len(pts)
    data_pts = np.empty((sz, 2), dtype=np.float64)
    for i in range(data_pts.shape[0]):
        data_pts[i,0] = pts[i,0,0]
        data_pts[i,1] = pts[i,0,1]
    # Perform PCA analysis
    mean = np.empty((0))
    mean, eigenvectors, eigenvalues = cv2.PCACompute2(data_pts, mean)
    # Store the center of the object
    cntr = (int(mean[0,0]), int(mean[0,1]))

    angle = atan2(eigenvectors[0,1], eigenvectors[0,0]) # orientation in radians
    cont = dict()
    cont['mean'] = mean
    cont['EVec'] = eigenvectors
    cont['EVal'] = eigenvalues
    cont['angle'] = angle

    return cont

def worker(frame):
    # Convert image to grayscale
    if len(frame) == 2:
        gray = cv2.cvtColor(frame[1], cv2.COLOR_BGR2GRAY)
    else:
        gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)


    # Convert image to binary
    _, bw = cv2.threshold(gray, 50, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)
    _, contours, _ = cv2.findContours(bw, cv2.RETR_LIST, cv2.CHAIN_APPROX_NONE)
    L = []
    for i, c in enumerate(contours):
        # Calculate the area of each contour
        area = cv2.contourArea(c);
        # Ignore contours that are too small or too large
        if area < 1e2 or 1e5 < area:
            continue
        # Find the orientation of each shape
        L.append(getOrientation(c))
    return L

def vectorToString(arr): #Da formato al paquete
    word = "[\n"
    for cont in arr:
        word = word + "mean: ("
        x,y = cont['mean'].shape
        for i in range(x):
            word = word + str(cont['mean'][0][i])
            if i < (x-1):
                word = word + ", "
        word = word + ")\n"
        x,y = cont['EVec'].shape
        word = word + "Eigenvectors: ["
        for j in range(y):
            word = word + "["
            for i in range(x):
                word = word + str(cont['EVec'][j][i])
                if i < (x-1):
                    word = word + ", "
            if j < (y-1):
                word = word + "]\n"
        word = word + "]]\n"
        x,y = cont['EVal'].shape
        word = word + "Eigenvalues: ("
        for j in range(y):
            word = word + str(cont['mean'][j][0])
            if j < (y-1):
                word = word + ", "
        word = word + ")\n"
        word = word + "]\n"
    #print(word)
    return word

def on_publish(client,userdata,result):
    pass


#----Setup mqtt----#
cap = cv2.VideoCapture(0) #setting the camera
cap.set(cv2.CAP_PROP_FRAME_WIDTH, 320)
cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 240)
broker = "192.168.0.21" # "10.112.13.232" #
port = 1883
client1 = paho.Client("control1")
client1.on_publish = on_publish
client1.connect(broker, port)

print("Starting video acquisition. Press 'Ctrl+C' to stop.")
start_time = time.time()
counterpackets = 0
try:
    while True:
        #time.sleep(0.01)
        ret, frame = cap.read()
        objects = worker(frame)
        counterpackets += 1
        ret = client1.publish("Camera", vectorToString(objects))

except KeyboardInterrupt:
    print("--- %s seconds ---" % (time.time() - start_time))
    print("--- %s packets ---" % (counterpackets))
