import time #se necesita para usar las funciones de tiempo
from subprocess import call #la necesitamos para la interrupcion de teclado
import RPi.GPIO as GPIO
import paho.mqtt.client as paho
#from pykalman import KalmanFilter
import numpy as np
from numpy.linalg import inv

class KalmanF:
    def __init__(self, F, B, H, Q, R, XkU=np.array([0]), PkU=np.array([0])):
        self.Kk = None
        self.Xk = None
        self.Pk = None
        self.XkU = XkU
        self.PkU = PkU
        self.F = F
        self.B = B
        self.H = H
        self.Q = Q
        self.R = R

    def Predict(self):
        self.Xk = np.matmul(self.F,self.XkU)
        aux = self.F.transpose()
        self.Pk = np.matmul(np.matmul(self.F,self.PkU), aux) + self.Q
        return (self.Xk,self.Pk)

    def Update(self, Zk):
        aux = np.matmul(np.matmul(self.H, self.Pk), self.H.transpose()) + self.R
        self.Kk = np.matmul(np.matmul(self.Pk, self.H), inv(aux))
        aux2 = Zk - np.matmul(self.H, self.Xk)
        self.XkU = self.Xk + np.matmul(self.Kk, aux2)
        self.PkU = self.Pk - np.matmul(np.matmul(self.Kk, self.H), self.Pk)
        return self.XkU


broker="192.168.0.21"
port=1883
def on_publish(client,userdata,result):
    pass

client1 = paho.Client("control1")
client1.on_publish = on_publish
client1.connect(broker,port)

GPIO.setmode(GPIO.BOARD) #Queremos usar la numeracion de la placa

#Definimos los dos pines del sensor que hemos conectado: Trigger y Echo
Trig = 7
Echo = 11

#Hay que configurar ambos pines del HC-SR04
GPIO.setup(Trig, GPIO.OUT)
GPIO.setup(Echo, GPIO.IN)

print("Sensor Ultrasonico")

#Kalman Filter parameters
state = np.array([[1]])
feedback = np.array([[0]])
measurement = np.array([[1]])

Q = np.array([[10**-5]]) #Process Noise Covariance Matrix
R = np.array([[2.92*10**-3]]) #Covariance Matrix of the Measurement Noise

kf = KalmanF(state, feedback, measurement, Q, R, XkU=np.array([[0]]), PkU=np.array([[0]])) #F, B, H, Q, R
kf.Predict()

try:
    while True:
        GPIO.output(Trig, False) #apagamos el pin Trig
        time.sleep(0.01)

        GPIO.output(Trig, True) #encendemos el pin Trig
        time.sleep(0.00001)
        GPIO.output(Trig, False) #y lo volvemos a apagar
        start = time.time()

        #empezaremos a contar el tiempo cuando el pin Echo se encienda
        while GPIO.input(Echo) == 0:
            start = time.time()
        while GPIO.input(Echo) == 1:
            end = time.time()
            #if((end-start) > 0.001):
            #    break
        #print(end-start)
        #Calculo de la distancia
        #Para calcular la distancia entre el sensor y el objeto
        #se usa la formula Distancia = velocidad * tiempo
        #Como la onda toma la mitad del tiempo en llegar, dividimos el resultado en 2
        #La velocidad del sonido en un medio de aire al nivel del mar es aproximadamente 34300 cm/s
        #por lo tanto multiplicamos la duracion del pulso por 17150 para obtener la distancia
        duracion = end-start
        medida = duracion*17150
        #print(medida)
        #KalmanFilter:
        if(medida < 400):
            filtrado = kf.Update(np.array([[medida]]))
            kf.Predict()
        else:
        #El sensor no esta disenado para detectar distancias mayores a 400 cm
        #Al no usar KF evitamos que la medicion del estado anterior se altere
            filtrado = 401
        #print(float(medida))
        #Publishing result:
        if(filtrado == 401):
            ret = client1.publish("debug", "%.2f - out of bounds" %(medida))
        else:
            ret = client1.publish("debug", "%.2f - %.2f" %(medida, filtrado))

except KeyboardInterrupt:
    GPIO.cleanup()
